import numpy as np
import matplotlib.pyplot as plt


def rxy():
    k = np.random.choice(4, p=[.7, .2, .05, .05])
    if k == 0:
        # Generate a sample from the boundary of the face.
        r = np.random.normal(loc=1, scale=0.07)
        theta = np.random.uniform(0, 2 * np.pi)
        x = r * np.cos(theta)
        y = r * np.sin(theta)
    elif k == 1:
        # Generate a sample from the smile on the face.
        r = np.random.normal(loc=0.5, scale=0.07)
        theta = np.random.uniform(-np.pi * 4 / 5, -np.pi / 5)
        x = r * np.cos(theta)
        y = r * np.sin(theta)
    elif k == 2:
        # Generate a sample from the right eye.
        s = np.random.normal(loc=[0.35, 0.3], scale=0.08)
        x, y = s[0], s[1]
    elif k == 3:
        # Generate a sample from the left eye.
        s = np.random.normal(loc=[-0.35, 0.3], scale=0.08)
        x, y = s[0], s[1]
    return x, y


n = 5000
xy = [rxy() for _ in range(n)]
x, y = [x for x, _ in xy], [y for _, y in xy]

fig, ((ax_x, dummy), (ax_xy, ax_y)) = plt.subplots(2, 2, figsize=(4, 4), sharex='col', sharey='row',
                                                   gridspec_kw={'height_ratios': [1, 2], 'width_ratios': [2, 1]})
dummy.remove()
ax_xy.scatter(x, y, s=3, alpha=.1)
ax_x.hist(x, density=True, bins=60)
ax_y.hist(y, density=True, bins=60, orientation='horizontal')
plt.show()
