import numpy as np
import scipy.stats

x = [4.3, 2.8, 3.9, 4.1, 4.5, 3.3]  # Dataset without x5.
x_5 = 9  # The fifth sample in the original dataset.
samples = 10000  # Number of samples to use.
prior_mu, prior_sigma = 0, 5  # Parameters for the Normal prior.
p_cauchy = 0.01
likelihood_sigma = 0.5


def prx(x, mu, h_5):
    p1 = scipy.stats.norm.pdf(x, loc=mu, scale=likelihood_sigma)
    p2 = scipy.stats.cauchy.pdf(x)
    # P(x_1, .. , x_4, x_6, x_7 | mu)
    p_others = np.prod((1 - p_cauchy) * p1 + p_cauchy * p2)
    # P(x_5 | h_5, mu)
    if h_5 == 'cauchy':
        p_5 = scipy.stats.cauchy.pdf(x_5)
    else:
        p_5 = scipy.stats.norm.pdf(x_5, loc=mu, scale=likelihood_sigma)
    return p_others * p_5


# Generate the samples.
mu = np.random.normal(loc=prior_mu, scale=prior_sigma, size=samples)
h_5 = np.random.choice(['cauchy', 'normal'], p=[p_cauchy, 1 - p_cauchy], replace=True, size=samples)

# Compute P(h_9, mu | x1, ... , xn).
posterior = np.array([prx(x, m, h) for m, h in zip(mu, h_5)])

# Marginalise out mu.
w_cauchy = np.sum(posterior[h_5 == 'cauchy'])
w_normal = np.sum(posterior[h_5 == 'normal'])

# Normalise to cancel out the constant.
print(w_cauchy / (w_cauchy + w_normal))
