import numpy as np
import pandas

climate = pandas.read_csv('https://www.cl.cam.ac.uk/teaching/2021/DataSci/data/climate.csv')
climate['t'] = climate.yyyy + (climate.mm - 1) / 12
climate['temp'] = (climate.tmin + climate.tmax) / 2

# Let's look at Oxford, which has longer records.
df = climate.loc[(climate.station == 'Oxford') & (~pandas.isna(climate.temp))]
t, temp = df['t'], df['temp']

# Sample from the prior distributions.
samples = 100000
alpha = np.random.uniform(8, 12, size=samples)
# Introduce a dependence between beta1 and beta2.
beta_magnitude = np.random.uniform(6, 8, size=samples)
beta_angle = np.random.uniform(0, 2 * np.pi, size=samples)
beta_1 = beta_magnitude * np.cos(beta_angle)
beta_2 = beta_magnitude * np.sin(beta_angle)

gamma = np.random.uniform(-1 / 100, 7 / 100, size=samples)
sigma = np.random.normal(loc=1.41, scale=0.1, size=samples)


# Compute weights
def logpr(temp, a, b1, b2, gamma, t, sigma):
    pred = a + b1 * np.sin(2 * np.pi * t) + b2 * np.cos(2 * np.pi * t) + gamma * (t - 2000)
    n = len(temp)
    return - n / 2 * np.sqrt(2 * np.pi * (sigma ** 2)) - np.sum((temp - pred) ** 2) / (2 * (sigma ** 2))


w = np.array([logpr(temp, a, b1, b2, g, t, s) for (a, b1, b2, g, s) in zip(alpha, beta_1, beta_2, gamma, sigma)])
# Log-exp trick in order to avoid underflow.
w = np.exp(w - max(w))
w = w / np.sum(w)

# Find a 95% confidence interval (see section 6.2 of lecture notes)
i = np.argsort(gamma)
gamma, w = gamma[i], w[i]
F = np.cumsum(w)
print(f'[{gamma[F < 0.025][-1]}, {gamma[F > 0.975][0]}]')
