import numpy as np
import scipy.stats

samples = 100000  # The number of samples to take.
p_healthy = 0.99  # Probability of a patient being healthy.
mu_healthy, sigma_healthy = 0, 2.1  # Parameters of the Normal distribution for the test result of healthy patient.
sigma_sick = 3.2  # The st. deviation of the Normal distribution for the test result of sick patient.
prior_mu_sick, prior_sigma_sick = 5, 3  # Parameters of the Normal prior distribution for mean of sick patients.
x_31 = 8.8  # The result of the unknown patient.
x_mean, n = 10.3, 30  # The mean result for the 30 sick patients.

# Collecting the samples. We need to sample both parameters: health status and mean for sick patients.
health_status = np.random.choice(['healthy', 'sick'], p=[p_healthy, 1.0 - p_healthy], replace=True, size=samples)
mu = np.random.normal(loc=prior_mu_sick, scale=prior_sigma_sick, size=samples)

# The likelihood of the 31rst patient P(x_31 | mu, h) for the two cases of h.
likelihood_x31_healthy = scipy.stats.norm.pdf(x=x_31, loc=mu_healthy, scale=sigma_healthy)
likelihood_x31_sick = scipy.stats.norm.pdf(x=x_31, loc=mu, scale=sigma_sick)

# P(x_{1:30} | mu, h) * P(x_31 | mu, h)
# Note: This is proportional to the posterior (the prior terms are added through sampling).
weights = np.exp(-(x_mean - mu) ** 2 / (2 * n * sigma_sick ** 2)) \
          * np.where(health_status == 'healthy', likelihood_x31_healthy, likelihood_x31_sick)

# By summing for both healthy and sick, we can marginalise out "mu".
posterior_healthy = np.sum(weights[health_status == 'healthy']) / np.sum(weights)
print((posterior_healthy, 1 - posterior_healthy))